--******************************************************************************************************************************
--
-- Description: This core is used in place of bpi_flash as an interface stub for the configuration flash memory.  The stub can
--				be used to conserve FPGA resources when there is no desire to program the flash from software.
--
-- Rev History: 07-14-2015:	Initial Release.
--
--******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

Library UNISIM;
use UNISIM.vcomponents.all;
library UNIMACRO;
use unimacro.Vcomponents.all;

use work.model_specific_package.all;

entity bpi_flash is
	port (
    -- Reset and clock.                                                                                      
	rst 			: in std_logic;								-- Active high asynchronous reset.
	user_clk 		: in std_logic;								-- System clock.
	clk200 			: in std_logic;								-- 200 MHz clock from an external oscillator.

	-- Connections to command/status registers in the BAR0 module.
	bpi_cmd_bus		: in std_logic_vector(32 downto 0);			-- Command bus that carries instructions and data to program the BPI flash.
	bpi_cmd_full	: out std_logic; 							-- Logic one indicates the BPI command FIFO cannot accept another burst.
	bpi_reply_bus	: out std_logic_vector(32 downto 0); 		-- Reply bus that returns data and errors from the BPI flash.
	bpi_reply_rd	: in std_logic; 							-- Logic one pops the BPI reply FIFO to the next value.
	clk20_locked	: out std_logic; 							-- Logic one indicates MMCM locked.
	
	--	The following signals connect to external devices through FPGA pins.
	bpi_a			: out std_logic_vector(BPI_ADD-1 downto 0);	-- Address bus.
	bpi_rs			: out std_logic_vector(1 downto 0);			-- Revision address.
	bpi_rs_disable	: out std_logic; 							-- Revision select disable.
	bpi_d_in		: in std_logic_vector(15 downto 0);			-- Data input (read) bus.
	bpi_d_out		: out std_logic_vector(15 downto 0);		-- Data output (write) bus.
	bpi_d_tristate	: out std_logic;							-- Active high data bus tristate.
	bpi_adv_b		: out std_logic; 							-- Active low address valid.
	bpi_fcs_b		: out std_logic; 							-- Active low chip enable.
	bpi_fwe_b		: out std_logic; 							-- Active low write enable.
	bpi_foe_b		: out std_logic; 							-- Active low output enable.
	bpi_vpp			: out std_logic 							-- Active high enable voltage for program and erase operations.
	);
end bpi_flash;

architecture structural of bpi_flash is

	type PROGRAM_STATE is (idle, enable, revision, address, write, read1, read2, read3, pop);	--	Type definitions for the state machine.
	signal bpi_state 	: PROGRAM_STATE;			-- Current state.
	signal command		: std_logic_vector(31 downto 0);	-- Command bus that carries instructions and data to program the flash.
	signal command_rd_en: std_logic;						-- Command bus read enable.
	signal command_vld	: std_logic;						-- Logic one indicates a valid command on the bus.
	signal reply		: std_logic_vector(31 downto 0);	-- Reply bus that returns data and errors from the BPI flash.
	signal reply_wr_en	: std_logic;						-- Reply bus write enable.
	signal reply_vld	: std_logic;						-- Logic one indicates a valid reply on the bus.
	signal reply_full	: std_logic;						-- Logic zero indicates that the reply FIFO can accept another burst.
	signal clkfbout		: std_logic;						-- MMCM feedback output.
	signal clkfbin		: std_logic;						-- MMCM feedback output on a global clock buffer.
	signal clkout0		: std_logic;						-- MMCM input clock divided by 10.
	signal clk20		: std_logic;						-- 20 MHz clock on a global clock bufffer.
	
begin

--******************************************* ASSIGN CONSTANTS TO ALL OUTPUT PORTS *********************************************

	bpi_cmd_full <= '0';
	bpi_reply_bus <= (others => '0');
	clk20_locked <= '1';

	bpi_a <= (others => '0');
	bpi_rs <= (others => '0');
	bpi_rs_disable <= '1';
	bpi_d_out <= bpi_d_in;
	bpi_d_tristate <= '0';
	bpi_adv_b <= '1';
	bpi_fcs_b <= '1';
	bpi_fwe_b <= '1';
	bpi_foe_b <= '1';
	bpi_vpp <= '0';    
	
end structural;
