--*******************************************************************************************************************************
--
-- Description: This model measures PCI throughput and maximum latency performance.
--					
-- Rev History: 01-10-2018:	Replaced asynchronous reset with synchronous reset.
--				08-04-2016:	Modifed the primary ports to match the actual number of DMA channels present.
-- 				07-14-2015:	Substituted IP generated counter with macro equivalent.
--				08-15-2014:	Eliminated the throughput measurement reset when the performance register is modified.
--				08-15-2014:	Corrected channel index error updating the maximum latency register.
--				05-28-2014:	Inital release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

Library UNISIM;
use UNISIM.vcomponents.all;
library UNIMACRO;
use unimacro.Vcomponents.all;

use work.model_specific_package.all;
use work.dma_package.all;
use work.transceiver_package.all;

entity pci_performance is
port (
    -- Clocks and control.
	clk 			: in std_logic;		-- System clock.

    -- DMA events and status.
	dma_wr			: in std_logic;						-- Logic one indicates that the PLDA_EZDMA2 core is supplying data from a DMA read request.
	dma_rd			: in std_logic;						-- Logic one indicates that the PLDA_EZDMA2 core is consuming data from a DMA write request.
	read_completion	: in std_logic_vector(1 downto 0);	-- Activity flags for dma_wrchannel(0)and dma_rdchannel(0).
	dma_reqchannel	: in CH_TYPE_1;						-- Activity flags for dma_wrchannel and dma_rdchannel benchmarks.
	dma_control		: in DMA_TYPE_6;					-- DMA control settings for the PLDA EZDMA2 core.

    -- Performance measurement registers.
	pci_measurement	: in std_logic_vector(1 downto 0);	-- Logic one clears the PCI throughput (0) or latency (1) register.
	pci_throughput	: out std_logic_vector(63 downto 0);-- Report the measured PCI DMA throughput in bytes/sec (transmitter=63:32, receiver=31:0).
	pci_latency		: out std_logic_vector(63 downto 0)	-- Report the maximum PCI DMA latency in 4 ns clock periods (transmitter=63:32, receiver=31:0).
);
end pci_performance;

architecture structural of pci_performance is

    signal qtr_second		: std_logic_vector(31 downto 0);	-- One second counter based on the 250 MHz PCI clock.
	signal qtr_second_mark	: std_logic;						-- Mark to indicate the completion of a one second timer interval.
	signal transactions		: std_logic_vector(63 downto 0);	-- Transaction count on the transmitter (63:32) and receiver (31:0).
	signal latency			: std_logic_vector(63 downto 0);	-- Latency timer on the transmitter (63:32) and receiver (31:0).
	signal dma_rx			: std_logic;						-- Logic one indicates an active DMA write transaction.
	signal dma_tx			: std_logic;						-- Logic one indicates an active DMA read transaction.
    signal latency_timer	: CH_TYPE_32;						-- Counter to time latency response based on the 250 MHz PCI clock.
	signal latency_enable	: CH_TYPE_1;						-- Logic one enable the DMA latency counter.
	signal latency_update	: unsigned(7 downto 0);				-- Logic one indicates that a new maximum value was measured on that channel.
	signal update_index		: integer;							-- Integer equivalent of latency_update bit position.
	signal latency_clear	: CH_TYPE_1;						-- Edge detect the PCI measurement enable.
	
begin

--************************************************** THROUGHPUT MEASUREMENTS ***************************************************

	process(clk)
	begin
		if rising_edge(clk) then
			-- Clear the counters anytime a new value is written to the throughput performance register.
			if pci_measurement(0) = '1' then
				qtr_second_mark <= '1';
				
			-- Report the performance in units of bytes/sec every quarter second (250 x 10^6 / 4 clock periods).
			elsif qtr_second = x"03B9ACA0" then
				-- Clear the counters on quarter second intervals.
				qtr_second_mark <= '1';
				-- AXI_WIDTH = 64: Eight bytes transferred per count (x8) and the measurement is made in a quarter second interval (x4).
				-- AXI_WIDTH = 128: Sixteen bytes transferred per count (x16) and the measurement is made in a quarter second interval (x4).
				-- Save the current receiver (DMA write) transaction count.
				pci_throughput(31 downto 4+(AXI_WIDTH/64)) <= transactions(27-(AXI_WIDTH/64) downto 0);
				-- Save the current tranmitter (DMA read) transaction count.
				pci_throughput(63 downto 36+(AXI_WIDTH/64)) <= transactions(59-(AXI_WIDTH/64) downto 32);
			else
				qtr_second_mark <= '0';
			end if;
		end if;
	end process;
	
	-- Set the pci_througput measurement LSBs to zero.
	pci_throughput(3+(AXI_WIDTH/64) downto 0) <= (others => '0');
	pci_throughput(35+(AXI_WIDTH/64) downto 32) <= (others => '0');

    -- Qualify each DMA transaction and do not count read completions on channel zero.
    dma_rx <= dma_rd and not(read_completion(0));
    dma_tx <= dma_wr and not(read_completion(1));
   
	-- Count a one second interval based on the 250 MHz PCI clock.
	QTR_SECOND_CNT : COUNTER_LOAD_MACRO
		generic map (DEVICE => "7SERIES", COUNT_BY => x"000000000001", WIDTH_DATA => 32)
		port map (Q => qtr_second, CLK => clk, CE => '1', DIRECTION => '1', LOAD => '0', LOAD_DATA => (others => '0'), RST => qtr_second_mark);
			
	-- Count the number of receiver transactions over a one second interval.
	RX_TRANSACTIONS : COUNTER_LOAD_MACRO
		generic map (DEVICE => "7SERIES", COUNT_BY => x"000000000001", WIDTH_DATA => 32)
		port map (Q => transactions(31 downto 0), CLK => clk, CE => dma_rx, DIRECTION => '1', LOAD => '0', LOAD_DATA => (others => '0'), RST => qtr_second_mark);
		
	-- Count the number of transmitter transactions over a one second interval.
	TX_TRANSACTIONS : COUNTER_LOAD_MACRO
		generic map (DEVICE => "7SERIES", COUNT_BY => x"000000000001", WIDTH_DATA => 32)
		port map (Q => transactions(63 downto 32), CLK => clk, CE => dma_tx, DIRECTION => '1', LOAD => '0', LOAD_DATA => (others => '0'), RST => qtr_second_mark);
		
--**************************************************** LATENCY MEASUREMENTS ****************************************************

	-- Perform a measurement on each receiver channel.
	RX_LATENCY : if RX_CHANNEL_COUNT > 0 generate
		-- Create a timer for each receiver channel instantiated.
		RX_DMA : for i in 1 to zero_to_one(RX_CHANNEL_COUNT) generate
			process(clk)
			begin
				if rising_edge(clk) then
					-- Start the timer at zero when a DMA request is issued to the PLDA EZDMA2 core.
					if dma_control(i) /= "000000" then
						latency_enable(i) <= '1';
						latency_clear(i) <= '1';
					-- Stop the timer when the first DMA transfer is detected on the channel.
					elsif dma_rd = '1' and dma_reqchannel(i) = '1' and latency_enable(i) = '1' then
						latency_enable(i) <= '0';
						-- Record the latency value if it is the largest measured.
						if latency_timer(i) > latency(31 downto 0) then
							latency_update(i-1) <= '1';
						end if;
					-- Release the clear and suspend updates while timing the event.
					else
						latency_clear(i) <= '0';
						latency_update(i-1) <= '0';
					end if;
				end if;
			end process;
			
			-- Count the number of 250 MHz PCI clock periods between a DMA request and the first transaction.
			RX_LATENCY_CNT : COUNTER_LOAD_MACRO
				generic map (DEVICE => "7SERIES", COUNT_BY => x"000000000001", WIDTH_DATA => 32)
				port map (Q => latency_timer(i), CLK => clk, CE => latency_enable(i), DIRECTION => '1', LOAD => '0', LOAD_DATA => (others => '0'), RST => latency_clear(i));

		end generate;
	end generate;

	-- Perform a measurement on each transmitter channel.
	TX_LATENCY : if TX_CHANNEL_COUNT > 0 generate
		-- Create a timer for each transmitter channel instantiated.
		TX_DMA : for i in RX_CHANNEL_COUNT+1 to TX_CHANNEL_COUNT+RX_CHANNEL_COUNT generate
			process(clk)
			begin
				if rising_edge(clk) then
					-- Start the timer at zero when a DMA request is issued to the PLDA EZDMA2 core.
					if dma_control(i) /= "000000" then
						latency_enable(i) <= '1';
						latency_clear(i) <= '1';
					-- Stop the timer when the first DMA transfer is detected on the channel.
					elsif dma_wr = '1' and dma_reqchannel(i) = '1' and latency_enable(i) = '1' then
						latency_enable(i) <= '0';
						-- Record the latency value if it is the largest measured.
						if latency_timer(i) > latency(63 downto 32) then
							latency_update(i-1) <= '1';
						end if;
					-- Release the clear and suspend updates while timing the event.
					else
						latency_clear(i) <= '0';
						latency_update(i-1) <= '0';
					end if;
				end if;
			end process;
			
			-- Count the number of 250 MHz PCI clock periods between a DMA request and the first transaction.
			TX_LATENCY_CNT : COUNTER_LOAD_MACRO
				generic map(DEVICE => "7SERIES", COUNT_BY => x"000000000001", WIDTH_DATA => 32)
				port map(Q => latency_timer(i), CLK => clk, CE => latency_enable(i), DIRECTION => '1', LOAD => '0', LOAD_DATA => (others => '0'), RST => latency_clear(i));

		end generate;
	end generate;

    -- Tie off latency update bits for unused channels.
	NO_CH : if TX_CHANNEL_COUNT+RX_CHANNEL_COUNT < 8 generate
		NO_UPDATE : for i in TX_CHANNEL_COUNT+RX_CHANNEL_COUNT+1 to 8 generate
			latency_update(i-1) <= '0';
		end generate;
	end generate;

	-- Convert bit position to corresponding integer.
	process (latency_update)
	begin
		case latency_update is
			when x"01" => update_index <= 1;
			when x"02" => update_index <= 2;
			when x"04" => update_index <= 3;
			when x"08" => update_index <= 4;
			when x"10" => update_index <= 5;
			when x"20" => update_index <= 6;
			when x"40" => update_index <= 7;
			when x"80" => update_index <= 8;
			when others => update_index <= 1;
		end case;
	end process;
		
	process(clk)
	begin
		if rising_edge(clk) then
			-- Reset the recorded latency to begin a new measurement.
			if pci_measurement(1) = '1' then
				latency <= (others => '0');
			-- Update the recorded latency if a new measurement is larger than the stored value.
			else
				-- Only one DMA channel is active at any time, so there can never be more than one update pending.
				if to_integer(latency_update) /= 0 then
					-- Update the maximum transmitter latency.
					if update_index > RX_CHANNEL_COUNT-1 then
						latency(63 downto 32) <= latency_timer(update_index);
					-- Update the maximum receiver latency.
					else
						latency(31 downto 0) <= latency_timer(update_index);
					end if;
				end if;
			end if;
		end if;
	end process;
	
	-- Assign the latency register value to the output port.
	pci_latency <= latency;

end structural;
