--*******************************************************************************************************************************
--
-- Description: This module performs the handshaking required to transfer a single period enable bit between two different clock
--				domains.  The target clock domain can run at any arbitrary frequency that is faster or slower than the source
--				clock domain.
--				Note: It will also translate the rising edge of a step input enable to a single period output enable.
--
-- Rev History: 01-08-2018:	Inital release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

entity sync_enable is
port (
    -- Source signals.
	source_srst 	: in std_logic;			-- Active high synchronous reset in the source clock domain.
	source_clk 		: in std_logic;			-- Clock connected to the source domain.
	source_enable	: in std_logic;			-- Logic one indicates that the source is sending an enable to the target.

    -- Target signals.
	target_srst 	: in std_logic;			-- Active high synchronous reset in the target clock domain.
	target_clk 		: in std_logic;			-- Clock connected to the target domain.
	target_enable	: out std_logic			-- Active high single period enable produced in the target clock domain.
);
end sync_enable;

architecture structural of sync_enable is

	signal target_post		: std_logic;					-- Post an enable to the target clock domain.
	signal target_reclock	: std_logic_vector(2 downto 0);	-- Reclock the enable bit in the target clock domain.
	
begin

--****************************************** POST AN ENABLE TO THE TARGET CLOCK DOMAIN *****************************************

process (source_clk)
    begin
        if rising_edge(source_clk) then
        
			if source_srst = '1' then
			
				target_post <= '0';
				
			else
				
				-- Detect a request from the source to send an enable to the target clock domain.
				if source_enable = '1' then
					-- Set a status bit to indicate that an operation is in progress and use it to cross clock domains.
					-- This register latency also ensures that any data in the source domain is stable before the target enable is generated.
					target_post <= '1';
				-- Detect the return handshake from the target clock domain to clear the status bit.
				elsif target_reclock(2) = '1' then
					-- Clear the status bit.
					target_post <= '0';
				end if;
        		
			end if;
        end if;
    end process;

--********************************** GENERATE A SINGLE PERIOD ENABLE IN THE TARGET CLOCK DOMAIN ********************************

process (target_clk)
    begin
        if rising_edge(target_clk) then
               
			if target_srst = '1' then
			
				target_reclock <= (others => '0');
				target_enable <= '0';
				
			else
				
				-- Register the target post bit for three clocks to ensure stability in the target clock domain.
				target_reclock(0) <= target_post;
				target_reclock(1) <= target_reclock(0);
				target_reclock(2) <= target_reclock(1);
			
				-- Rising edge detect the target post bit.
				if target_reclock(2) = '0' and target_reclock(1) = '1' then
					-- Activate the enable for one period in the target clock domain.
					target_enable <= '1';
				else
					-- Deactivate the enable in the target clock domain.
					target_enable <= '0';
				end if;
       	
			end if;
        end if;
    end process;
    
end structural;
